/***************************************************************************
 *   Copyright (C) 2004 by Michael Moritz                                  *
 *   mimo@restoel.net                                                      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef __CONFIGREADER_H
#define __CONFIGREADER_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <iosfwd>
#include <stdexcept>
#include <map>
#include <string>
/**
 * @author Michael Moritz

 *@brief reads configuration file

 *The configuration file has this syntax:
 * @code
 * prefix_key=value
 * @endcode
 * 		
 * @c prefix is used to group values and keys, it can be empty.	
 * For example:
 * @code
 * dbtype=mysql
 * db_host=localhost
 * db_username=greylist
 * @endcode
 * 	
 * Currently the following prefixes and keys are recognised:
 * 	- mode:	the operational mode of gps! 
 *			@warning Set this explicitly
 * 			to anything different from @e init if you want gps
 * 			to return anything other than action=dunno.
 * 			Using @e mode=normal is recommended for the future. 
 * 			Default: init
 * 	- dbtype: specify the database backend type for DBI 
 * 			this requires the selected DBD driver installed
 * 			This has so far only been tested with mysql.
 * 			Default: mysql 
 * 	- timeout:timeout in seconds before a triplet is moved to
 * 			registered state @ref RecordStatus.
 * 			Default: 3600
 * 	- db_*: database parameters passed to DBI. With mysql this
 * 			group consists of [example value]:
 * @code
 * db_host=[localhost]
 * db_username=[greylist]
 * db_password=[secret]
 * db_dbname=[greylist]
 * @endcode
 * 
**/
class ConfigReader
{
public:
	ConfigReader(std::istream &is) throw (std::exception);
	typedef std::map<std::string,std::string>StringMap;
	const StringMap& get(const std::string& prefix,bool dontthrow=false) const throw (std::exception);
	const std::string &getVal(const std::string& prefix,const std::string& key) const throw (std::exception);
	
	const std::string getValOrDef(const std::string& prefix,const std::string& key,const std::string& def) const throw (std::exception);
	
private:
	typedef std::map<std::string,StringMap>ConfigMap;
	ConfigMap _map;
	static StringMap s_emptyMap;
};
#endif

