/***************************************************************************
 *   Copyright (C) 2004 by Michael Moritz                                  *
 *   mimo@restoel.net                                                      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef __WLDB_H
#define __WLDB_H
/**
	@brief WLDB database backend for database whitelist
	@author Michael Moritz
*/
#include <exception>
#include <string>
#include <syslog.h>
#include "triplet.h"
#include "db.h"
#include "wlmodule.h"
#include "dbdefs.h"
#include "pmatcher.h"

class WLDB
{
public:
	WLDB(class DB& db, 
		bool initmode,
		const WLStaticDef& staticDef) throw(std::exception)
	: _db(db),
	  _staticDef(staticDef)
	{
		if(initmode) {
			_db.createTable(_staticDef);
		}
	}
	bool check(const class Triplet& triplet) throw(std::exception) {
		switch(_staticDef._compareMode) {
			case WlModule::CM_exactMatch:
				return _db.exactMatch( "wl",_staticDef,triplet );
			case WlModule::CM_patternMatch:
				return patternMatch( "wl", triplet );
			default:
				return false;
		}
	}	
	~WLDB() {};
protected:
	///in most cases it will be better to use dbcached for patterns instead of this
	bool patternMatch(const std::string logprefix, const Triplet& triplet)
	{
		dbi_result result = _db.executeSQL(_staticDef._searchTable);
		if(!result) //table does not exist, error has been logged
			return false;
		// this is required in older versions of libdbi -- otherwise no fieldnames
		unsigned numrows = dbi_result_get_numrows(result);
		if(numrows == 0) {
			dbi_result_free(result);
			return false;
		}
		dbi_result_next_row(result);
		unsigned numfields = dbi_result_get_numfields(result);
		//cout << "numfields: " << numfields << endl;
		unsigned commentIdx = numfields;
		unsigned primaryIdx = numfields;
		for(unsigned i=0;i < numfields;i++) {
			const char* colname = dbi_result_get_field_name(result,i+1);
			//cout << i << ": " << colname << endl;
			if(strcmp(colname,_staticDef._primaryKey) == 0)
				primaryIdx = i;
			else 
				if(strcmp(colname,_staticDef._commentField) == 0)
					commentIdx = i;
//			else
//				_heading.insert(Heading::value_type(colname,i));
		}
		if((primaryIdx >= numfields) || (commentIdx >= numfields)) { 
			//this means the table layout has changed--check dbdefs.cpp
			syslog(LOG_WARNING,"WlDb no primary/comment idx in table %s",_staticDef._tableName);
			dbi_result_free(result);
			return false;
		}
		PatternMatcher pmatcher(triplet);
		for(unsigned i=0;i < numrows;i++) {
			const char *pattern = dbi_result_get_string_idx(result,primaryIdx+1);
//			std::cout << "pattern: " << pattern << std::endl;
			if(pmatcher.match(pattern)) {
				const char *comment = dbi_result_get_string_idx(result,commentIdx+1);
				syslog(LOG_INFO,"%s %s: %s -> %s, %s: %s",
					logprefix.c_str(),_staticDef._tableName,triplet.getSender().c_str(),
					triplet.getRecipient().c_str(),triplet.getClientAddress().c_str(),comment);
				dbi_result_free(result);
				return true;
			}
			if(dbi_result_next_row(result) == 0)
				break;
		}
		dbi_result_free(result);
		return false;	
	}
private:
	class DB& _db;	
	const WLStaticDef& _staticDef;
};

#endif
